#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include "getfem_mesh.h"
#include "getfem_mesh_fem.h"
#include "getfem_interpolation.h"

using namespace std;

int main()
{

	//////////////////////////////////////////////////////////////////////////////////////////////
	//This block loads the meshes from the disk and associates their mesh_fem.
	getfem::mesh *atommesh = new getfem::mesh();
	try
	{
		atommesh->read_from_file( "./atommeshin.txt" );
	}
	catch(exception e)
	{
		cout << "Error reading atommeshin.txt file. Program will now abort." << endl;
		terminate();
	};
	getfem::mesh_fem atommesh_fem( *atommesh );
	atommesh_fem.set_finite_element( getfem::fem_descriptor("FEM_PK(3,1)") );

	getfem::mesh *largemesh = new getfem::mesh();
	try
	{
		largemesh->read_from_file( "./largemeshin.txt" );
	}
	catch(exception e)
	{
		cout << "Error reading largemeshin.txt file. Program will now abort." << endl;
		terminate();
	};
	getfem::mesh_fem largemesh_fem( *largemesh );
	largemesh_fem.set_finite_element( getfem::fem_descriptor("FEM_PK(3,1)") );
	//////////////////////////////////////////////////////////////////////////////////////////////

	cout << "Atommesh's number of points:" << atommesh->nb_points() << endl;
	cout << "Atommesh_fem number of dofs:" << atommesh_fem.nb_dof() << endl;

	////////////////////////////////////////////////////////////////////////
	//Creating a vector of data that we'll try to interpolate
	std::vector<double> ax( atommesh_fem.nb_dof() );
	for (unsigned long i = 0; i < ax.size(); i++)
		ax[i] = 5.555;	
	////////////////////////////////////////////////////////////////////////

	////////////////////////////////////////////////////////////////////////
	//Interpolating the accelerations from the atomic mesh to the large mesh	
	gmm::row_matrix< gmm::rsvector<double> > N( 1, ax.size() );
	std::vector<double> V( 1 );
	getfem::mesh_trans_inv mti( atommesh_fem.linked_mesh() );
	unsigned long number_of_sucess = 0;
	unsigned long nodecounter = largemesh->nb_points();
	do
	{
		//For each node in the large mesh, I'll try to interpolate the data from the atomic mesh
		nodecounter--;
		N.clear_mat();
		mti.clear();
		mti.add_point( largemesh->points()[nodecounter] );			
		getfem::interpolation( atommesh_fem, mti, ax, V, N, 1, false);
		gmm::mult( N, ax , V);
		
		if ( V[0] > 1 )
			number_of_sucess++;
		
		//During debug, you can remove the comment from the line below in order to see what happens at each node of the largemesh.
		//cout << "At the node located in the position:" << largemesh->points()[nodecounter] << ", we have this interpolation:" << V[0] << endl;
	}while(nodecounter);	
	////////////////////////////////////////////////////////////////////////

	////////////////////////////////////////////////////////////////////////
	//Let's see how many times it worked.
	cout << "Number of sucessfull interpolations:" << number_of_sucess << " out of " << largemesh->nb_points() << " attempts.\n";  
	////////////////////////////////////////////////////////////////////////

	delete atommesh;
	delete largemesh;
}
